﻿using Plane;
using Plane.Collections;
using Plane.FormationCreator.Formation;
using Plane.Logging;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.CommandWpf;
using System;
using System.Collections.Generic;
using System.Collections.Concurrent;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using Plane.Communication;
using Plane.Copters;
using Plane.Windows.Messages;
using Plane.FormationCreator.Properties;
using System.IO;
using Plane.CommunicationManagement;
using Plane.Protocols;
using Microsoft.Practices.ServiceLocation;
using System.Windows.Media;

namespace Plane.FormationCreator.ViewModels
{
    public class ConnectViewModel : ViewModelBase
    {
        public ConnectViewModel(CopterManager copterManager)
        {
            _copterManager = copterManager;
        }

        private CopterManager _copterManager;

        private const int PORT = 5250;

        // 不严谨，稍微验证一下。
        private Regex _ipListRegex = new Regex(@"^(?:" + Environment.NewLine + @")*\d{1,3}(?:\.\d{1,3}){3}(?:" + Environment.NewLine + @"\d{1,3}(?:\.\d{1,3}){3})*(?:" + Environment.NewLine + @")*$", RegexOptions.Singleline | RegexOptions.Compiled);

        //        private string _IPs = @"192.168.1.11
        //192.168.1.12
        //192.168.1.13
        //192.168.1.14
        //192.168.1.15
        //192.168.1.16
        //192.168.1.17
        //192.168.1.18
        //192.168.1.19
        //192.168.1.20
        //";
        private string _IPs = Settings.Default.IPs;
        public string IPs
        {
            get { return _IPs; }
            set { Set(nameof(IPs), ref _IPs, value); }
        }

        private bool _IsProcessing;
        public bool IsProcessing
        {
            get { return _IsProcessing; }
            set { Set(nameof(IsProcessing), ref _IsProcessing, value); }
        }

        private ICommand _ConnectCommand;
        public ICommand ConnectCommand
        {
            get
            {
                return _ConnectCommand ?? (_ConnectCommand = new RelayCommand<string>(async connectionType =>
                {
                    await ConnectAsync();
                }
                ));
            }
        }

        private ICommand _SendCommand;
        public ICommand SendCommand
        {
            get
            {
                return _SendCommand ?? (_SendCommand = new RelayCommand(async () =>
                {
                    await SendCommandAsync();
                }
                ));
            }
        }

//         private ICommand _WriteCommand;
//         public ICommand WriteCommand
//         {
//             get
//             {
//                 return _WriteCommand ?? (_WriteCommand = new RelayCommand(async () =>
//                 {
//                     await WriteCommandAsync();
//                 }
//                 ));
//             }
//         }

        private ICommand _WriteIdCommand;
        public ICommand WriteIdCommand
        {
            get
            {
                return _WriteIdCommand ?? (_WriteIdCommand = new RelayCommand(async () =>
                {
                    await WriteIdCommandAsync();
                }
                ));
            }
        }

        private ICommand _CommWriteMissionCommand;
        public ICommand CommWriteMissionCommand
        {
            get
            {
                return _CommWriteMissionCommand ?? (_CommWriteMissionCommand = new RelayCommand(async () =>
                {
                    var _flightTaskManager = ServiceLocator.Current.GetInstance<FlightTaskManager>();
                    if (_flightTaskManager.OriginLat == 0 && _flightTaskManager.OriginLng == 0)
                    {
                        Alert.Show("作为参照的原点未设置，无法写入相对位置！", "提示");
                        return;
                    }

                    int coptercount = _copterManager.Copters.Count;
                    int taskcount = _flightTaskManager.Tasks.Count;
                    //  bool havewritefault = true;
                    if (taskcount < 2) return;

                    for (int i = 0; i < coptercount; i++)
                    {

                        ///写航线开始
                        //var missions = new IMission[taskcount];  //不要起飞任务但增加一个起飞后低空航点，见起飞任务,再增加一个降落低空航点，见降落部分
                        var missions = new List<IMission>();
                        int missindex = 0;
                        ICopter copter = _copterManager.Copters[i];
                        if (copter is PLCopter)
                        {
                            for (int j = 0; j < _flightTaskManager.Tasks.Count; j++)
                            {

                                switch (_flightTaskManager.Tasks[j].TaskType)
                                {
                                    case FlightTaskType.TakeOff:
                                        //计算起飞需要的时间,5秒是上升加速和最后稳定时间
                                        int takeofftime = (int)(_flightTaskManager.Tasks[j + 1].SingleCopterInfos[i].TargetAlt / 2.5 + 5);

                                        missions.Add(Mission.CreateTakeoffMission(
                                            1,//1,
                                            5,//takeofftime,
                                            _flightTaskManager.Tasks[j + 1].SingleCopterInfos[i].TargetLat - _flightTaskManager.OriginLat,
                                            _flightTaskManager.Tasks[j + 1].SingleCopterInfos[i].TargetLng - _flightTaskManager.OriginLng,
                                            _flightTaskManager.Tasks[j + 1].SingleCopterInfos[i].TargetAlt)
                                         );

                                        //要起飞任务

                                        break;

                                    case FlightTaskType.FlyTo:

                                        foreach (LEDInfo ledInfo in _flightTaskManager.Tasks[j].SingleCopterInfos[i].LEDInfos)
                                        {
                                            Color color = (Color)ColorConverter.ConvertFromString("#" + ledInfo.LEDRGB);
                                            IMission LEDMission = Mission.CreateLEDControlMission(
                                                    (int)(ledInfo.Delay * 10),
                                                    ledInfo.LEDMode,
                                                    ledInfo.LEDRate,
                                                    0,   //ledInfo.LEDTimes,
                                                    color.R,
                                                    color.G,
                                                    color.B
                                                    );
                                            missions.Add(LEDMission);
                                        }
                                        missions.Add(Mission.CreateWaypointMission(

                                                _flightTaskManager.Tasks[j].LoiterTime,
                                                _flightTaskManager.Tasks[j].FlytoTime,
                                                _flightTaskManager.Tasks[j].SingleCopterInfos[i].TargetLat - _flightTaskManager.OriginLat,
                                                _flightTaskManager.Tasks[j].SingleCopterInfos[i].TargetLng - _flightTaskManager.OriginLng,

                                                _flightTaskManager.Tasks[j].SingleCopterInfos[i].TargetAlt)
                                            );

                                        break;
                                    case FlightTaskType.Loiter:

                                        break;
                                    case FlightTaskType.ReturnToLand:

                                        missions.Add(Mission.CreateReturnToLaunchMission());
                                        break;

                                    case FlightTaskType.Land:
                                        missions.Add(Mission.CreateLandMission(5));
                                        break;
                                    default:
                                        break;

                                }

                            }
                            await WriteCommandAsync(copter, missions);
                        }
                        
                        //_copterManager.Copters[i].WriteMissionListAsync(missions);
                        /*
                        if (!result)
                        {

                            Alert.Show($"飞机：{_copterManager.Copters[i].Name} 任务写入失败!");
                            return;
                        }
                        */

                    }
                    Alert.Show($"所有任务写入成功!");
                }));
            }
        }


        private async Task WriteIdCommandAsync()
        {
            await commModule.GeneratePacketAsync(3, (byte)Protocols.MavComm.COMMANDTYPE4);
        }


        private async Task SendCommandAsync()
        {
            Protocols.MavComm.comm_set_mav_count mavCount = new Protocols.MavComm.comm_set_mav_count();
            mavCount.mav_count = 200;
            await commModule.GeneratePacketAsync(0, (byte)Protocols.MavComm.COMM_SET_MAV_COUNT, mavCount);
        }

        private async Task WriteCommandAsync(ICopter copter, List<IMission> missions)
        {
            List<MAVLink.mavlink_mission_item_t> mission_list = new List<MAVLink.mavlink_mission_item_t>();
            foreach (IMission mission in missions)
            {
                var frame = mission.Command == FlightCommand.Waypoint && mission.Sequence == 0 ? MAVLink.MAV_FRAME.GLOBAL : MAVLink.MAV_FRAME.GLOBAL_RELATIVE_ALT;

                var req = new MAVLink.mavlink_mission_item_t();

                PLCopter plCopter = (PLCopter)copter;
                req.target_system = plCopter._internalCopter.sysid;
                req.target_component = plCopter._internalCopter.compid;

                req.command = (byte)mission.Command;

                req.current = 0;
                req.autocontinue = 1;

                req.frame = (byte)frame;
                if (mission.Command == FlightCommand.LEDControl)
                {
                    req.x = mission.R;
                    req.y = mission.G;
                    req.z = mission.B;
                }
                else
                {
                    req.x = (float)mission.Latitude;
                    req.y = (float)mission.Longitude;
                    req.z = mission.Altitude;
                }

                req.param1 = mission.Param1;
                req.param2 = mission.Param2;
                req.param3 = mission.Param3;
                req.param4 = mission.Param4;

                req.seq = mission.Sequence;
                mission_list.Add(req);
            }

            await commModule.GeneratePacketAsyncNew(short.Parse(copter.Name), (byte)Protocols.MavComm.COMM_WRITE_MISSION, mission_list.ToArray());
        }


        CommModule commModule = new CommModule();

        private async Task ConnectAsync()
        {
            await commModule.ConnectAsync();
        }       

        private Action _closeWindowCallback;
        public void SetCloseWindowAction(Action closeWindowAction)
        {
            _closeWindowCallback = closeWindowAction;
        }
        private void CloseWindow()
        {
            if (_closeWindowCallback == null) return;
            var dispatcher = App.Current.Dispatcher;
            if (dispatcher.CheckAccess())
            {
                _closeWindowCallback();
            }
            else
            {
                dispatcher.InvokeAsync(() =>
                {
                    _closeWindowCallback.Invoke();
                });
            }
        }
    }
}
