﻿using Plane.Communication;
using Plane.FormationCreator.Formation;
using Plane.Windows.Messages;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.CommandWpf;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using Microsoft.Win32;
using Microsoft.Practices.ServiceLocation;
using System.IO;

namespace Plane.FormationCreator.ViewModels
{
    public class MainViewModel : ViewModelBase
    {
        public MainViewModel(CopterListViewModel copterListViewModel)
        {
            _copterListViewModel = copterListViewModel;

            Plane.Windows.Messages.Message.Configure(showAction: msg => this.Message = msg);

            this.SwitchVelocityModeButtonContent = GetSwitchVelocityModeButtonContent();

            AppEx.Current.PropertyChanged += AppEx_PropertyChanged;
        }

        private CopterListViewModel _copterListViewModel;
        public CopterListViewModel CopterListViewModel { get { return _copterListViewModel; } }
        private FlightTaskManager _flightTaskManager = ServiceLocator.Current.GetInstance<FlightTaskManager>();
        public AppEx AppEx { get; } = AppEx.Current;

        private void AppEx_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case nameof(AppEx.IsInFastMode):
                    this.SwitchVelocityModeButtonContent = GetSwitchVelocityModeButtonContent();
                    break;
                default:
                    break;
            }
        }

        private string _SwitchVelocityModeButtonContent;
        public string SwitchVelocityModeButtonContent
        {
            get { return _SwitchVelocityModeButtonContent; }
            set { Set(nameof(SwitchVelocityModeButtonContent), ref _SwitchVelocityModeButtonContent, value); }
        }

        private string _Message;
        public string Message
        {
            get { return _Message; }
            set {
                    Set(nameof(Message), ref _Message, value);
                    _MessageList.Add(value);
            }
        }

        private List<string> _MessageList = new List<string>();
        public string Messages
        {
            get { return string.Join("\r\n", _MessageList.ToArray()); }
        }

        private ICommand _RestartListeningCommand;
        public ICommand RestartListeningCommand
        {
            get
            {
                return _RestartListeningCommand ?? (_RestartListeningCommand = new RelayCommand(() =>
                {
                    TcpServerConnectionManager.Instance.StopListening();
                    if (TcpServerConnectionManager.Instance.StartListening())
                    {
                        Alert.Show("已重启对连接请求的监听。");
                    }
                    else
                    {
                        Alert.Show("网络连接不正常，无法启动监听。");
                    }

                    UdpServerConnectionManager.Instance.StopReceiving();
                    UdpServerConnectionManager.Instance.StartReceiving();
                }));
            }
        }

        private ICommand _SwitchVelocityModeCommand;
        public ICommand SwitchVelocityModeCommand
        {
            get
            {
                return _SwitchVelocityModeCommand ?? (_SwitchVelocityModeCommand = new RelayCommand(() =>
                {
                    AppEx.Current.IsInFastMode = !AppEx.Current.IsInFastMode;
                }));
            }
        }

        private ICommand _SwitchAppModeCommand;
        public ICommand SwitchAppModeCommand
        {
            get
            {
                return _SwitchAppModeCommand ?? (_SwitchAppModeCommand = new RelayCommand<AppMode>(mode =>
                {
                    AppEx.Current.AppMode = mode;
                }));
            }
        }

        private ICommand _ShowOrHideModifyTaskViewCommand;
        public ICommand ShowOrHideModifyTaskViewCommand
        {
            get
            {
                return _ShowOrHideModifyTaskViewCommand ?? (_ShowOrHideModifyTaskViewCommand = new RelayCommand(() =>
                {
                    AppEx.Current.ShowModifyTaskView = !AppEx.Current.ShowModifyTaskView;
                }));
            }
        }


        private ICommand _ExportTasksCommand;
        public ICommand ExportTasksCommand
        {
            get
            {
                return _ExportTasksCommand ?? (_ExportTasksCommand = new RelayCommand(() =>
                {
                    if (_flightTaskManager.OriginLat == 0 && _flightTaskManager.OriginLng == 0)
                    {
                        Alert.Show("作为参照的原点未设置，无法导出相对位置！", "提示");
                        return;
                    }
                    var exportedText = _flightTaskManager.ExportTasks();
                    var dialog = new SaveFileDialog
                    {
                        DefaultExt = "fcg",
                        Filter = "编队飞行任务 (*.fcg)|*.fcg"
                    };
                    if (dialog.ShowDialog() == true)
                    {
                        File.WriteAllText(dialog.FileName, exportedText);
                    }
                }));
            }
        }


        private int _txtStarindex = 0;
        public int txtStarindex
        {
            get { return _txtStarindex; }
            set { Set(nameof(txtStarindex), ref _txtStarindex, value); }
        }

        private int _txtendindex = 0;
        public int txtendindex
        {
            get { return _txtendindex; }
            set { Set(nameof(txtendindex), ref _txtendindex, value); }
        }

        private ICommand _ImportTasksCommand;
        public ICommand ImportTasksCommand
        {
            get
            {
                return _ImportTasksCommand ?? (_ImportTasksCommand = new RelayCommand(() =>
                {
                    if (_flightTaskManager.OriginLat == 0 && _flightTaskManager.OriginLng == 0)
                    {
                        Alert.Show("作为参照的原点未设置，无法导入相对位置！", "提示");
                        return;
                    }
                    var dialog = new OpenFileDialog
                    {
                        DefaultExt = "fcg",
                        Filter = "编队飞行任务 (*.fcg)|*.fcg"
                    };
                    if (dialog.ShowDialog() == true)
                    {
                        int _startindex = txtStarindex;
                        int _endindex = txtendindex;

                        var tasksText = File.ReadAllText(dialog.FileName);
                        if ((txtStarindex == 0) && (txtendindex == 0))
                            _flightTaskManager.ImportTasks(tasksText);
                        else
                        {
                            _endindex = txtendindex;
                            if (_startindex == 0)
                                _startindex = 1;
                            if (_endindex == 0)
                                _endindex = _startindex;
                            _flightTaskManager.ImportTasksindex(tasksText, _startindex, _endindex);
                        }


                    }
                }));
            }
        }

        private string GetSwitchVelocityModeButtonContent()
            => AppEx.Current.IsInFastMode ? "高速模式" : "低速模式";
    }
}
